﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;
using DlhSoft.Windows.Data;

namespace GanttChartViewResourceDragDropSample
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        // The moving item thumb, source resource, and source resource item recorded when a drag operation begins.
        private Thumb itemThumb;
        private Resource sourceResource;
        private ResourceItem sourceResourceItem;

        // The hovering task recorded during the drag operation.
        private Task hoveringTask;

        // When a drag operation is started initialize the appropriate fields.
        private void ItemThumb_DragStarted(object sender, DragStartedEventArgs e)
        {
            // Record the item thumb, source resource, and source resource item.
            itemThumb = e.Source as Thumb;
            sourceResource = itemThumb != null ? itemThumb.DataContext as Resource : null;
            sourceResourceItem = sourceResource != null ? sourceResource.Item as ResourceItem : null;
            // Select the dragging resource to visually indicate the operation source.
            ResourceListView.SelectedItem = sourceResourceItem;
            e.Handled = true;
        }

        // When a drag operation is in progress.
        private void ItemThumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            // Determine the hovering task.
            ItemsControl ganttChartViewItemsControl = GanttChartView.BaseItemsControl;
            Point mousePosition = Mouse.GetPosition(ganttChartViewItemsControl);
            HitTestResult hitTestResult = VisualTreeHelper.HitTest(ganttChartViewItemsControl, mousePosition);
            FrameworkElement hoveringFrameworkElement = hitTestResult != null ? hitTestResult.VisualHit as FrameworkElement : null;
            hoveringTask = hoveringFrameworkElement != null ? hoveringFrameworkElement.DataContext as Task : null;
            if (hoveringTask != null && HasAssignment(hoveringTask, sourceResourceItem))
                hoveringTask = null;
            FrameworkElement hoveringItemFrameworkElement = hoveringTask != null && !hoveringTask.HasChildren ? GanttChartView.BaseItemsControl.ItemContainerGenerator.ContainerFromItem(hoveringTask) as FrameworkElement : null;
            // Set the mouse cursor as appropriate during the drag and drop operation.
            itemThumb.Cursor = hoveringItemFrameworkElement != null ? Cursors.Pen : Cursors.Cross;
            e.Handled = true;
        }

        private static bool HasAssignment(Task task, ResourceItem resourceItem)
        {
            if (task == null)
                return false;
            foreach (Assignment assignment in task.Assignments)
            {
                Resource resource = assignment.Resource;
                if (resource != null && resource.Item == resourceItem)
                    return true;
            }
            return false;
        }

        // When a drag operation is completed.
        private void ItemThumb_DragCompleted(object sender, DragCompletedEventArgs e)
        {
            // Clear the mouse cursor.
            itemThumb.ClearValue(CursorProperty);
            // Determine the destination task item.
            TaskItem destinationTaskItem = hoveringTask != null ? hoveringTask.Item as TaskItem : null;
            if (destinationTaskItem != null && sourceResourceItem != null)
            {
                // Assign the source resource item to the destination task item.
                destinationTaskItem.Assignments.Add(new AssignmentItem(sourceResourceItem));
            }
            // Clear fields.
            itemThumb = null;
            sourceResource = null;
            sourceResourceItem = null;
            hoveringTask = null;
            e.Handled = true;
        }
    }
}
